/*
 * Copyright 2013 (c) Anna Schumaker.
 *
 * DO NOT INCLUDE THIS FILE DIRECTLY.  THIS IS A TEMPLATE DEFINITION FILE
 * AND ONLY MEANT TO BE INCLUDED BY include/database.h!
 */
#ifndef OCARINA_DATABASE_HPP
#define OCARINA_DATABASE_HPP

template <class T>
DatabaseEntry<T> :: DatabaseEntry()
	       : valid(false)
{
}



template <class T, class U>
Database<T, U> :: Database(std::string filepath)
	     : _size(0), file(filepath, FILE_TYPE_DATA)
{
}

template <class T, class U>
Database<T, U> :: ~Database()
{
}

template <class T, class U>
void Database<T, U> :: save()
{
	if (file.open(OPEN_WRITE) == false)
		return;

	file << db.size() << std::endl;
	for (unsigned int i = 0; i < db.size(); i++) {
		file << db[i].valid << " ";
		if (db[i].valid == true)
			db[i].write(file);
		file << std::endl;
	}

	file.close();
}

template <class T, class U>
void Database<T, U> :: load()
{
	unsigned int db_size;
	if (file.open(OPEN_READ) == false)
		return;

	file >> db_size;
	db.resize(db_size);
	for (unsigned int i = 0; i < db_size; i++) {
		file >> db[i].valid;
		if (db[i].valid == true) {
			db[i].read(file);
			keys.insert(std::pair<std::string, unsigned int>(db[i].primary_key(), i));
			_size++;
		}
	}

	file.close();
}

#ifdef CONFIG_DEBUG
template <class T, class U>
void Database<T, U> :: clear()
{
	db.clear();
	keys.clear();
	_size = 0;
}

template <class T, class U>
void Database<T, U> :: print()
{
	:: print("Allocated rows: %u\n", db.size());
	:: print("Valid rows: %u\n", _size);
	for (unsigned int i = 0; i < db.size(); i++) {
		if (db[i].valid == true) {
			:: print("db[%u] = ", i);
			db[i].print();
			:: print("\n");
		}
	}
}

template <class T, class U>
void Database<T, U> :: print_keys()
{
	std::map<const std::string, unsigned int>::iterator it;
	:: print("Found keys:");
	for (it = keys.begin(); it != keys.end(); it++)
		:: print(" %s", it->first.c_str());
	:: print("\n");
}
#endif /* CONFIG_DEBUG */

template <class T, class U>
unsigned int Database<T, U> :: insert(T val)
{
	unsigned int id;
	typename std::map<U, unsigned int>::iterator it;

	it = keys.find(val.primary_key());
	if (it != keys.end())
		return it->second;

	/*
	 * Check primary key stuff here
	 */
	id = db.size();
	db.push_back(val);
	keys.insert(std::pair<U, unsigned int>(val.primary_key(), id));
	db[id].valid = true;
	_size++;
	return id;
}

template <class T, class U>
void Database<T, U> :: remove(unsigned int id)
{
	keys.erase(db[id].primary_key());
	db[id].valid = false;
	_size--;
}

template <class T, class U>
unsigned int Database<T, U> :: size()
{
	return _size;
}

template <class T, class U>
unsigned int Database<T, U> :: num_rows()
{
	return db.size();
}

template <class T, class U>
unsigned int Database<T, U> :: first()
{
	for (unsigned int i = 0; i < db.size(); i++) {
		if (db[i].valid == true)
			return i;
	}

	return db.size();
}

template <class T, class U>
unsigned int Database<T, U> :: last()
{
	if (_size == 0)
		return db.size();

	for (unsigned int i = db.size() - 1; i >= 0; i--) {
		if (db[i].valid == true)
			return i;
	}

	return db.size();
}

template <class T, class U>
unsigned int Database<T, U> :: next(unsigned int id)
{
	for (unsigned int i = id + 1; i < db.size(); i++) {
		if (db[i].valid == true)
			return i;
	}

	return db.size();
}

template <class T, class U>
unsigned int Database<T, U> :: find_index(const std::string &key)
{
	typename std::map<U, unsigned int>::iterator it;
	it = keys.find(key);
	if (it == keys.end())
		throw 0;
	return it->second;
}

template <class T, class U>
T &Database<T, U> :: find(const std::string &key)
{
	return db[find_index(key)];
}

template <class T, class U>
T &Database<T, U> :: operator[](unsigned int id)
{
	return db[id];
}

#endif /* OCARINA_DATABASE_HPP */
