/*
 * Copyright 2013 (c) Anna Schumaker.
 */
#include <deck.h>
#include <print.h>

#include <list>

static std::list<Playlist> playlist_deck;

void deck :: read(File &f)
{
	unsigned int num;
	std::list<Playlist>::iterator it;

	f >> num;
	playlist_deck.resize(num);

	for (it = playlist_deck.begin(); it != playlist_deck.end(); it++)
		it->read(f);
}

void deck :: write(File &f)
{
	std::list<Playlist>::iterator it;

	f << playlist_deck.size() << std :: endl;
	for (it = playlist_deck.begin(); it != playlist_deck.end(); it++) {
		it->write(f);
		f << std::endl;
	}
}

Playlist *deck :: create()
{
	playlist_deck.push_back(Playlist(PL_ENABLED));
	return &playlist_deck.back();
}

void deck :: remove(unsigned int id)
{
	std::list<Playlist>::iterator it = playlist_deck.begin();
	for (unsigned int i = 0; i < id; i++)
		it++;
	playlist_deck.erase(it);
}

Playlist *deck :: get(unsigned int id)
{
	std::list<Playlist>::iterator it = playlist_deck.begin();
	for (unsigned int i = 0; i < id; i++)
		it++;
	return &(*it);
}

void deck :: move(unsigned int old_pos, unsigned int new_pos)
{
	std::list<Playlist>::iterator it_old = playlist_deck.begin();
	std::list<Playlist>::iterator it_new = playlist_deck.begin();

	for (unsigned int i = 0; i < playlist_deck.size(); i++) {
		if (i < old_pos)
			it_old++;
		if (i < new_pos)
			it_new++;
	}

	if (new_pos > old_pos)
		it_new++;

	playlist_deck.splice(it_new, playlist_deck, it_old);
}

unsigned int deck :: next()
{
	unsigned int id = 0;
	std::list<Playlist>::iterator it;

	for (it = playlist_deck.begin(); it != playlist_deck.end(); it++) {
		if (it->get_flags() & PL_ENABLED) {
			id = it->next();
			if (it->size() == 0)
				playlist_deck.erase(it);
			return id;
		}
	}

	throw -1;
}

#ifdef CONFIG_DEBUG
void deck :: reset()
{
	playlist_deck.clear();
}

void deck :: print_info()
{
	unsigned int i = 0;
	std::list<Playlist>::iterator it;

	for (it = playlist_deck.begin(); it != playlist_deck.end(); it++) {
		print("deck[%u] = Playlist { size = %u, flags = %u }\n",
			i, it->size(), it->get_flags());
		i++;
	}
}
#endif /* CONFIG_DEBUG */
